import type { Handler } from "aws-lambda";
import { createRestAPIClient } from "masto";
import "dotenv/config";

// import Scraper from "../../utils/scraper";
import ScraperArticles from "../../utils/scraper-articles";
import RedisClient from "../../libs/redis-client";

import config from "../../config";
import LogLevels from "../../enums/log-levels";
import Emojis from "../../enums/emojis";

export const handler: Handler = async (event, context) => {
  const name = "La Tercera";
  const baseDomain = "https://www.latercera.cl"
  try {
    const redisClient = new RedisClient();
    const mastodon = createRestAPIClient({
      url: config.MASTODON_URL,
      accessToken: config.MASTODON_ACCESS_TOKEN
    });
    // const scraper = new Scraper();
    const scraperArticles = new ScraperArticles(name, {
      url: config.LATERCERA,
      articlesSelector: "article.card",
      titleSelector: "div.headline",
      contentSelector: "div.deck",
      linkSelector: "div.headline a",
      imageSelector: "div.image figure a picture img",
      authorSelector: "div.name a small",
      dateSelector: ""
    });

    const articles = await scraperArticles.getArticles();
    if (config.LOG_LEVEL === LogLevels.DEBUG) {
      console.log("Articles", articles);
    }

    const length = articles.length;
    for (let i = 0; i < length; i++) {
      const article = articles[i];
      const exists = await redisClient.retrieve(article.link);
      if (exists !== null) {
        continue;
      }

      const date = new Date(Date.now()).toLocaleDateString();
      const message = `${Emojis.NEWS} ${article.title} \n${baseDomain}${article.link}`;
      const mediaIds = [""];

      // Since Mastodon is able to read metas this is not necessary
      // const imgFile = await scraper.scrapeImage(article.image);
      // if (imgFile !== null) {
      //   const media = await mastodon.v2.media.create({ file: imgFile, description: article.content });
      //   mediaIds.push(media.id);
      // }

      await mastodon.v1.statuses.create({ status: message, mediaIds });
      await redisClient.store(article.link, date, { EX: 60 * 60 * 24 }); // EX: 24 hrs expiration
    }
    console.log(`Published ${length} new articles`);
  } catch (err: any) {
    console.log('An error has occurred\n')
    console.error(err.message);
    if (config.LOG_LEVEL === LogLevels.DEBUG) {
      console.debug("\nEvent\n");
      console.debug(event);
      console.debug("\nContext\n");
      console.debug(context);
    }
  }

  return "The End.";
};
